@@ -35,6 +35,13 @@
  *
  * This file has various different disc utility functions for getting
  * the media types and labels of discs.
+ *
+ * The functions in this file refer to MRLs, which are a special form
+ * of URIs used by xine to refer to things such as DVDs. An example of
+ * an MRL would be <example>dvd:///dev/scd0</example>, which is not a
+ * valid URI as far as, for example, GIO is concerned.
+ *
+ * The rest of the totem-pl-parser API exclusively uses URIs.
  **/
 
 #include "config.h"
@@ -51,23 +58,15 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
-#ifdef HAVE_HAL
-#include <libhal.h>
-#include <dbus/dbus.h>
-#endif
-
 #include "totem-disc.h"
+#include "totem-pl-parser.h"
 
 typedef struct _CdCache {
   /* device node and mountpoint */
   char *device, *mountpoint;
   GVolume *volume;
 
-#ifdef HAVE_HAL
-  LibHalContext *ctx;
-  /* If the disc is a media, have the UDI available here */
-  char *disc_udi;
-#endif
+  char **content_types;
 
   GFile *iso_file;
 
@@ -129,96 +128,139 @@ totem_resolve_symlink (const char *device, GError **error)
 }
 
 static gboolean
-cd_cache_get_dev_from_volumes (GVolumeMonitor *mon, const char *device,
-			      char **mountpoint, GVolume **v)
+cd_cache_get_best_mount_for_drive (GDrive *drive,
+				   char **mountpoint,
+				   GVolume **volume)
 {
+  GList *list, *l;
+  int rank;
+
+  rank = 0;
+  list = g_drive_get_volumes (drive);
+  for (l = list; l != NULL; l = l->next) {
+    GVolume *v;
+    GMount *m;
+    GFile *file;
+    int new_rank;
+
+    new_rank = 0;
+    v = l->data;
+
+    m = g_volume_get_mount (v);
+    if (m == NULL)
+      continue;
+
+    file = g_mount_get_root (m);
+    if (g_file_has_uri_scheme (file, "cdda"))
+      new_rank = 100;
+    else
+      new_rank = 50;
+
+    if (new_rank > rank) {
+      if (*mountpoint)
+	g_free (*mountpoint);
+      if (*volume)
+	g_object_unref (volume);
+
+      *volume = g_object_ref (v);
+      *mountpoint = g_file_get_path (file);
+      rank = new_rank;
+    }
+
+    g_object_unref (file);
+    g_object_unref (m);
+  }
+
+  g_list_foreach (list, (GFunc) g_object_unref, NULL);
+  g_list_free (list);
+
+  return rank > 0;
+}
+
+static gboolean
+cd_cache_get_dev_from_volumes (GVolumeMonitor *mon, const char *device,
+			      char **mountpoint, GVolume **volume)
+{
+  GList *list, *l;
   gboolean found;
-  GVolume *volume = NULL;
-  GList *list, *or;
 
   found = FALSE;
+  list = g_volume_monitor_get_connected_drives (mon);
+  for (l = list; l != NULL; l = l->next) {
+    GDrive *drive;
+    char *ddev, *resolved;
 
-  for (or = list = g_volume_monitor_get_volumes (mon);
-       list != NULL; list = list->next) {
-    char *pdev, *pdev2;
+    drive = l->data;
+    ddev = g_drive_get_identifier (drive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+    if (ddev == NULL)
+      continue;
+    resolved = totem_resolve_symlink (ddev, NULL);
+    g_free (ddev);
+    if (resolved == NULL)
+      continue;
 
-    volume = list->data;
-    if (!(pdev = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE)))
-      continue;
-    pdev2 = totem_resolve_symlink (pdev, NULL);
-    if (!pdev2) {
-      g_free (pdev);
-      continue;
+    if (strcmp (resolved, device) == 0) {
+      found = cd_cache_get_best_mount_for_drive (drive, mountpoint, volume);
     }
-    g_free (pdev);
 
-    if (strcmp (pdev2, device) == 0) {
-      GMount *mount;
-
-      mount = g_volume_get_mount (volume);
-      if (mount) {
-	GFile *file;
-
-	file = g_mount_get_root (mount);
-	*mountpoint = g_file_get_path (file);
-	g_object_unref (file);
-	g_object_unref (mount);
-      }
-
-      found = TRUE;
-      g_object_ref (volume);
-      g_free (pdev2);
+    g_free (resolved);
+    if (found != FALSE)
       break;
-    }
-    g_free (pdev2);
   }
-  g_list_foreach (or, (GFunc) g_object_unref, NULL);
-  g_list_free (or);
 
-  *v = volume;
+  g_list_foreach (list, (GFunc) g_object_unref, NULL);
+  g_list_free (list);
+
+  if (found != FALSE)
+    return found;
+
+  /* Not in the drives? Look in the volumes themselves */
+  found = FALSE;
+  list = g_volume_monitor_get_volumes (mon);
+  for (l = list; l != NULL; l = l->next) {
+    GVolume *vol;
+    char *ddev, *resolved;
+
+    vol = l->data;
+    ddev = g_volume_get_identifier (vol, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+    if (ddev == NULL)
+      continue;
+    resolved = totem_resolve_symlink (ddev, NULL);
+    g_free (ddev);
+    if (resolved == NULL)
+      continue;
+
+    if (strcmp (resolved, device) == 0) {
+      found = TRUE;
+      *volume = g_object_ref (vol);
+    }
+
+    g_free (resolved);
+    if (found != FALSE)
+      break;
+  }
+
+  g_list_foreach (list, (GFunc) g_object_unref, NULL);
+  g_list_free (list);
 
   return found;
 }
 
-#ifdef HAVE_HAL
-static LibHalContext *
-cd_cache_new_hal_ctx (void)
+static gboolean
+cd_cache_has_content_type (CdCache *cache, const char *content_type)
 {
-  LibHalContext *ctx;
-  DBusConnection *conn;
-  DBusError error;
+  guint i;
 
-  ctx = libhal_ctx_new ();
-  if (ctx == NULL)
-    return NULL;
-
-  dbus_error_init (&error);
-  conn = dbus_bus_get_private (DBUS_BUS_SYSTEM, &error);
-
-  if (conn != NULL && !dbus_error_is_set (&error)) {
-    dbus_connection_set_exit_on_disconnect (conn, FALSE);
-    if (!libhal_ctx_set_dbus_connection (ctx, conn)) {
-      libhal_ctx_free (ctx);
-      return NULL;
-    }
-    if (libhal_ctx_init (ctx, &error))
-      return ctx;
+  if (cache->content_types == NULL) {
+    return FALSE;
   }
 
-  if (dbus_error_is_set (&error)) {
-    g_warning ("Couldn't get the system D-Bus: %s", error.message);
-    dbus_error_free (&error);
+  for (i = 0; cache->content_types[i] != NULL; i++) {
+    if (g_str_equal (cache->content_types[i], content_type) != FALSE)
+      return TRUE;
   }
-
-  libhal_ctx_free (ctx);
-  if (conn != NULL) {
-    dbus_connection_close (conn);
-    dbus_connection_unref (conn);
-  }
-
-  return NULL;
+  return FALSE;
 }
-#endif
 
 static char *
 cd_cache_local_file_to_archive (const char *filename)
@@ -251,23 +293,20 @@ cd_cache_new (const char *dev,
   char *mountpoint = NULL, *device, *local;
   GVolumeMonitor *mon;
   GVolume *volume = NULL;
-#ifdef HAVE_HAL
-  LibHalContext *ctx = NULL;
-#endif
+  GFile *file;
   gboolean found;
 
-  if (dev[0] == '/')
+  if (dev[0] == '/') {
     local = g_strdup (dev);
-  else {
-    GFile *file;
-
+    file = g_file_new_for_path (dev);
+  } else {
     file = g_file_new_for_commandline_arg (dev);
     local = g_file_get_path (file);
-    g_object_unref (file);
   }
 
   if (local == NULL) {
     /* No error, just no cache */
+    g_object_unref (file);
     return NULL;
   }
 
@@ -275,6 +314,8 @@ cd_cache_new (const char *dev,
     cache = g_new0 (CdCache, 1);
     cache->mountpoint = local;
     cache->is_media = FALSE;
+    cache->content_types = g_content_type_guess_for_tree (file);
+    g_object_unref (file);
 
     return cache;
   } else if (g_file_test (local, G_FILE_TEST_IS_REGULAR)) {
@@ -282,6 +323,8 @@ cd_cache_new (const char *dev,
     GError *err = NULL;
     char *archive_path;
 
+    g_object_unref (file);
+
     cache = g_new0 (CdCache, 1);
     cache->is_iso = TRUE;
     cache->is_media = FALSE;
@@ -310,8 +353,8 @@ cd_cache_new (const char *dev,
 	if (data.error) {
 	  g_propagate_error (error, data.error);
 	} else {
-	  g_set_error (error, 0, 0,
-		       _("Failed to mount %s"), cache->device);
+	  g_set_error (error, TOTEM_PL_PARSER_ERROR, TOTEM_PL_PARSER_ERROR_MOUNT_FAILED,
+		       _("Failed to mount %s."), cache->device);
 	}
 	cd_cache_free (cache);
 	return FALSE;
@@ -329,6 +372,8 @@ cd_cache_new (const char *dev,
     return cache;
   }
 
+  g_object_unref (file);
+
   /* We have a local device
    * retrieve mountpoint and volume from gio volumes */
   device = totem_resolve_symlink (local, error);
@@ -338,36 +383,28 @@ cd_cache_new (const char *dev,
   mon = g_volume_monitor_get ();
   found = cd_cache_get_dev_from_volumes (mon, device, &mountpoint, &volume);
   if (!found) {
-    g_set_error (error, 0, 0,
-	_("No media in drive for device '%s'"),
+    g_set_error (error, TOTEM_PL_PARSER_ERROR, TOTEM_PL_PARSER_ERROR_NO_DISC,
+	_("No media in drive for device '%s'."),
 	device);
     g_free (device);
     return NULL;
   }
 
-#ifdef HAVE_HAL
-  ctx = cd_cache_new_hal_ctx ();
-  if (!ctx) {
-    g_set_error (error, 0, 0,
-	_("Could not connect to the HAL daemon"));
-    g_free (device);
-    return NULL;
-  }
-#endif
-
   /* create struture */
   cache = g_new0 (CdCache, 1);
   cache->device = device;
   cache->mountpoint = mountpoint;
   cache->self_mounted = FALSE;
   cache->volume = volume;
-#ifdef HAVE_HAL
-  cache->disc_udi = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
-#endif
   cache->is_media = TRUE;
-#ifdef HAVE_HAL
-  cache->ctx = ctx;
-#endif
+
+  {
+    GMount *mount;
+
+    mount = g_volume_get_mount (volume);
+    cache->content_types = g_mount_guess_content_type_sync (mount, FALSE, NULL, NULL);
+    g_object_unref (mount);
+  }
 
   return cache;
 }
@@ -382,6 +419,8 @@ cd_cache_has_medium (CdCache *cache)
     return FALSE;
 
   drive = g_volume_get_drive (cache->volume);
+  if (drive == NULL)
+    return FALSE;
   retval = g_drive_has_media (drive);
   g_object_unref (drive);
 
@@ -398,7 +437,7 @@ cd_cache_open_device (CdCache *cache,
   }
 
   if (cd_cache_has_medium (cache) == FALSE) {
-    g_set_error (error, 0, 0,
+    g_set_error (error, TOTEM_PL_PARSER_ERROR, TOTEM_PL_PARSER_ERROR_NO_DISC,
 	_("Please check that a disc is present in the drive."));
     return FALSE;
   }
@@ -455,8 +494,8 @@ cd_cache_open_mountpoint (CdCache *cache,
       if (data.error) {
 	g_propagate_error (error, data.error);
       } else {
-	g_set_error (error, 0, 0,
-		     _("Failed to mount %s"), cache->device);
+	g_set_error (error, TOTEM_PL_PARSER_ERROR, TOTEM_PL_PARSER_ERROR_MOUNT_FAILED,
+		     _("Failed to mount %s."), cache->device);
       }
       return FALSE;
     } else {
@@ -489,20 +528,7 @@ cd_cache_free (CdCache *cache)
 {
   GMount *mount;
 
-#ifdef HAVE_HAL
-  if (cache->ctx != NULL) {
-    DBusConnection *conn;
-
-    conn = libhal_ctx_get_dbus_connection (cache->ctx);
-    libhal_ctx_shutdown (cache->ctx, NULL);
-    libhal_ctx_free(cache->ctx);
-    /* Close the connection before doing the last unref */
-    dbus_connection_close (conn);
-    dbus_connection_unref (conn);
-
-    g_free (cache->disc_udi);
-  }
-#endif /* HAVE_HAL */
+  g_strfreev (cache->content_types);
 
   if (cache->iso_file) {
     mount = g_file_find_enclosing_mount (cache->iso_file,
@@ -535,106 +561,17 @@ static TotemDiscMediaType
 cd_cache_disc_is_cdda (CdCache *cache,
 		       GError **error)
 {
-  TotemDiscMediaType type;
-
   /* We can't have audio CDs on disc, yet */
-  if (cache->is_media == FALSE)
+  if (cache->is_media == FALSE) {
     return MEDIA_TYPE_DATA;
+  }
   if (!cd_cache_open_device (cache, error))
     return MEDIA_TYPE_ERROR;
 
-#ifdef HAVE_HAL
-  {
-    DBusError error;
-    dbus_bool_t is_cdda;
+  if (cd_cache_has_content_type (cache, "x-content/audio-cdda") != FALSE)
+    return MEDIA_TYPE_CDDA;
 
-    if (libhal_device_property_exists (cache->ctx,
-				       cache->disc_udi, "volume.disc.has_audio", NULL) == FALSE)
-      return MEDIA_TYPE_DATA;
-
-    dbus_error_init (&error);
-
-    is_cdda = libhal_device_get_property_bool (cache->ctx,
-	cache->disc_udi, "volume.disc.has_audio", &error);
-    type = is_cdda ? MEDIA_TYPE_CDDA : MEDIA_TYPE_DATA;
-
-    if (dbus_error_is_set (&error)) {
-      g_warning ("Error checking whether the volume is an audio CD: %s",
-	  error.message);
-      dbus_error_free (&error);
-      return MEDIA_TYPE_ERROR;
-    }
-    return type;
-  }
-#else
-  {
-    GMount *mount;
-    GFile *root;
-
-    type = MEDIA_TYPE_DATA;
-
-    mount = g_volume_get_mount (cache->volume);
-    if (!mount)
-      return type;
-
-    root = g_mount_get_root (mount);
-    if (g_file_has_uri_scheme (root, "cdda"))
-      type = MEDIA_TYPE_CDDA;
-
-    g_object_unref (root);
-  }
-
-  return type;
-#endif
-}
-
-static gboolean
-cd_cache_file_exists (CdCache *cache, const char *subdir, const char *filename)
-{
-  char *path, *dir;
-  gboolean ret;
-
-  dir = NULL;
-
-  /* Check whether the directory exists, for a start */
-  path = g_build_filename (cache->mountpoint, subdir, NULL);
-  ret = g_file_test (path, G_FILE_TEST_IS_DIR);
-  if (ret == FALSE) {
-    char *subdir_low;
-
-    g_free (path);
-    subdir_low = g_ascii_strdown (subdir, -1);
-    path = g_build_filename (cache->mountpoint, subdir_low, NULL);
-    ret = g_file_test (path, G_FILE_TEST_IS_DIR);
-    g_free (path);
-    if (ret) {
-      dir = subdir_low;
-    } else {
-      g_free (subdir_low);
-      return FALSE;
-    }
-  } else {
-    g_free (path);
-    dir = g_strdup (subdir);
-  }
-
-  /* And now the file */
-  path = g_build_filename (cache->mountpoint, dir, filename, NULL);
-  ret = g_file_test (path, G_FILE_TEST_IS_REGULAR);
-  if (ret == FALSE) {
-    char *fname_low;
-
-    g_free (path);
-    fname_low = g_ascii_strdown (filename, -1);
-    path = g_build_filename (cache->mountpoint, dir, fname_low, NULL);
-    ret = g_file_test (path, G_FILE_TEST_IS_REGULAR);
-    g_free (fname_low);
-  }
-
-  g_free (dir);
-  g_free (path);
-
-  return ret;
+  return MEDIA_TYPE_DATA;
 }
 
 static TotemDiscMediaType
@@ -648,46 +585,10 @@ cd_cache_disc_is_vcd (CdCache *cache,
     return MEDIA_TYPE_ERROR;
   if (!cache->mountpoint)
     return MEDIA_TYPE_ERROR;
-#ifdef HAVE_HAL
-  if (cache->is_media != FALSE) {
-    DBusError error;
-    dbus_bool_t is_vcd;
 
-    if (libhal_device_property_exists (cache->ctx,
-				       cache->disc_udi, "volume.disc.is_vcd", NULL) != FALSE) {
-      dbus_error_init (&error);
-
-      is_vcd = libhal_device_get_property_bool (cache->ctx,
-						cache->disc_udi, "volume.disc.is_vcd", &error);
-
-      if (dbus_error_is_set (&error)) {
-	g_warning ("Error checking whether the volume is a VCD: %s",
-		   error.message);
-	dbus_error_free (&error);
-	return MEDIA_TYPE_ERROR;
-      }
-      if (is_vcd != FALSE)
-	return MEDIA_TYPE_VCD;
-    }
-    if (libhal_device_property_exists (cache->ctx,
-				       cache->disc_udi, "volume.disc.is_svcd", NULL) == FALSE)
-      return MEDIA_TYPE_DATA;
-
-    is_vcd = libhal_device_get_property_bool (cache->ctx,
-	cache->disc_udi, "volume.disc.is_svcd", &error);
-
-    if (dbus_error_is_set (&error)) {
-      g_warning ("Error checking whether the volume is an SVCD: %s",
-	  error.message);
-      dbus_error_free (&error);
-      return MEDIA_TYPE_ERROR;
-    }
-    return is_vcd ? MEDIA_TYPE_VCD : MEDIA_TYPE_DATA;
-  }
-#endif
-  /* first is VCD, second is SVCD */
-  if (cd_cache_file_exists (cache, "MPEGAV", "AVSEQ01.DAT") ||
-      cd_cache_file_exists (cache, "MPEG2", "AVSEQ01.MPG"))
+  if (cd_cache_has_content_type (cache, "x-content/video-vcd") != FALSE)
+    return MEDIA_TYPE_VCD;
+  if (cd_cache_has_content_type (cache, "x-content/video-svcd") != FALSE)
     return MEDIA_TYPE_VCD;
 
   return MEDIA_TYPE_DATA;
@@ -704,33 +605,8 @@ cd_cache_disc_is_dvd (CdCache *cache,
     return MEDIA_TYPE_ERROR;
   if (!cache->mountpoint)
     return MEDIA_TYPE_ERROR;
-#ifdef HAVE_HAL
-  if (cache->is_media != FALSE) {
-    DBusError error;
-    dbus_bool_t is_dvd;
 
-    if (libhal_device_property_exists (cache->ctx,
-				       cache->disc_udi, "volume.disc.is_videodvd", NULL) == FALSE)
-      return MEDIA_TYPE_DATA;
-
-    dbus_error_init (&error);
-
-    is_dvd = libhal_device_get_property_bool (cache->ctx,
-	cache->disc_udi, "volume.disc.is_videodvd", &error);
-
-    if (dbus_error_is_set (&error)) {
-      g_warning ("Error checking whether the volume is a DVD: %s",
-	  error.message);
-      dbus_error_free (&error);
-      return MEDIA_TYPE_ERROR;
-    }
-    return is_dvd ? MEDIA_TYPE_DVD : MEDIA_TYPE_DATA;
-  }
-#endif
-  if (cd_cache_file_exists (cache, "VIDEO_TS", "VIDEO_TS.IFO"))
-    return MEDIA_TYPE_DVD;
-  /* FIXME bug in libarchive? */
-  if (cd_cache_file_exists (cache, "VIDEO_TS", "VIDEO_TS.IFO;1"))
+  if (cd_cache_has_content_type (cache, "x-content/video-dvd") != FALSE)
     return MEDIA_TYPE_DVD;
 
   return MEDIA_TYPE_DATA;
@@ -782,17 +658,25 @@ totem_cd_dir_get_parent (const char *dir)
 /**
  * totem_cd_detect_type_from_dir:
  * @dir: a directory URI
- * @url: return location for the disc's MRL, or %NULL
+ * @mrl: return location for the disc's MRL, or %NULL
  * @error: return location for a #GError, or %NULL
  *
  * Detects the disc's type, given its mount directory URI. If
- * a string pointer is passed to @url, it will return the disc's
+ * a string pointer is passed to @mrl, it will return the disc's
  * MRL as from totem_cd_mrl_from_type().
  *
+ * Note that this function does synchronous I/O.
+ *
+ * If no disc is present in the drive, a #TOTEM_PL_PARSER_ERROR_NO_DISC
+ * error will be returned. On unknown mounting errors, a
+ * #TOTEM_PL_PARSER_ERROR_MOUNT_FAILED error will be returned. On other
+ * I/O errors, or if resolution of symlinked mount paths failed, a code from
+ * #GIOErrorEnum will be returned.
+ *
  * Return value: #TotemDiscMediaType corresponding to the disc's type, or #MEDIA_TYPE_ERROR on failure
  **/
 TotemDiscMediaType
-totem_cd_detect_type_from_dir (const char *dir, char **url, GError **error)
+totem_cd_detect_type_from_dir (const char *dir, char **mrl, GError **error)
 {
   CdCache *cache;
   TotemDiscMediaType type;
@@ -823,15 +707,15 @@ totem_cd_detect_type_from_dir (const char *dir, char **url, GError **error)
     }
   }
 
-  if (url == NULL) {
+  if (mrl == NULL) {
     cd_cache_free (cache);
     return type;
   }
 
   if (type == MEDIA_TYPE_DVD) {
-    *url = totem_cd_mrl_from_type ("dvd", cache->mountpoint);
+    *mrl = totem_cd_mrl_from_type ("dvd", cache->mountpoint);
   } else if (type == MEDIA_TYPE_VCD) {
-    *url = totem_cd_mrl_from_type ("vcd", cache->mountpoint);
+    *mrl = totem_cd_mrl_from_type ("vcd", cache->mountpoint);
   }
 
   cd_cache_free (cache);
@@ -842,25 +726,29 @@ totem_cd_detect_type_from_dir (const char *dir, char **url, GError **error)
 /**
  * totem_cd_detect_type_with_url:
  * @device: a device node path
- * @url: return location for the disc's MRL, or %NULL
+ * @mrl: return location for the disc's MRL, or %NULL
  * @error: return location for a #GError, or %NULL
  *
  * Detects the disc's type, given its device node path. If
- * a string pointer is passed to @url, it will return the disc's
+ * a string pointer is passed to @mrl, it will return the disc's
  * MRL as from totem_cd_mrl_from_type().
  *
+ * Note that this function does synchronous I/O.
+ *
+ * Possible error codes are as per totem_cd_detect_type_from_dir().
+ *
  * Return value: #TotemDiscMediaType corresponding to the disc's type, or #MEDIA_TYPE_ERROR on failure
  **/
 TotemDiscMediaType
 totem_cd_detect_type_with_url (const char *device,
-    			       char      **url,
+    			       char      **mrl,
 			       GError     **error)
 {
   CdCache *cache;
   TotemDiscMediaType type;
 
-  if (url != NULL)
-    *url = NULL;
+  if (mrl != NULL)
+    *mrl = NULL;
 
   if (!(cache = cd_cache_new (device, error)))
     return MEDIA_TYPE_ERROR;
@@ -877,7 +765,7 @@ totem_cd_detect_type_with_url (const char *device,
     /* crap, nothing found */
   }
 
-  if (url == NULL) {
+  if (mrl == NULL) {
     cd_cache_free (cache);
     return type;
   }
@@ -891,7 +779,7 @@ totem_cd_detect_type_with_url (const char *device,
 	str = cache->mountpoint ? cache->mountpoint : device;
       else
 	str = cache->device;
-      *url = totem_cd_mrl_from_type ("dvd", str);
+      *mrl = totem_cd_mrl_from_type ("dvd", str);
     }
     break;
   case MEDIA_TYPE_VCD:
@@ -902,7 +790,7 @@ totem_cd_detect_type_with_url (const char *device,
 	str = cache->mountpoint ? cache->mountpoint : device;
       else
 	str = cache->device;
-      *url = totem_cd_mrl_from_type ("vcd", str);
+      *mrl = totem_cd_mrl_from_type ("vcd", str);
     }
     break;
   case MEDIA_TYPE_CDDA:
@@ -911,9 +799,9 @@ totem_cd_detect_type_with_url (const char *device,
 
       dev = cache->device ? cache->device : device;
       if (g_str_has_prefix (dev, "/dev/") != FALSE)
-	*url = totem_cd_mrl_from_type ("cdda", dev + 5);
+	*mrl = totem_cd_mrl_from_type ("cdda", dev + 5);
       else
-	*url = totem_cd_mrl_from_type ("cdda", dev);
+	*mrl = totem_cd_mrl_from_type ("cdda", dev);
     }
     break;
   case MEDIA_TYPE_DATA:
@@ -921,7 +809,7 @@ totem_cd_detect_type_with_url (const char *device,
       type = MEDIA_TYPE_ERROR;
       /* No error, it's just not usable */
     } else {
-      *url = g_filename_to_uri (cache->mountpoint, NULL, NULL);
+      *mrl = g_strdup (cache->mountpoint);
     }
     break;
   default:
@@ -940,6 +828,8 @@ totem_cd_detect_type_with_url (const char *device,
  *
  * Detects the disc's type, given its device node path.
  *
+ * Possible error codes are as per totem_cd_detect_type_with_url().
+ *
  * Return value: #TotemDiscMediaType corresponding to the disc's type, or #MEDIA_TYPE_ERROR on failure
  **/
 TotemDiscMediaType
